@@ -10,7 +10,6 @@
 # https://github.com/yt-dlp/yt-dlp/blob/master/yt_dlp/cookies.py
 
 import binascii
-import contextlib
 import ctypes
 import logging
 import os
@@ -26,7 +25,7 @@ from . import aes, text, util
 
 
 SUPPORTED_BROWSERS_CHROMIUM = {
-    "brave", "chrome", "chromium", "edge", "opera", "vivaldi"}
+    "brave", "chrome", "chromium", "edge", "opera", "thorium", "vivaldi"}
 SUPPORTED_BROWSERS = SUPPORTED_BROWSERS_CHROMIUM | {"firefox", "safari"}
 
 logger = logging.getLogger("cookies")
@@ -147,7 +146,8 @@ def load_cookies_chrome(cookiejar, browser_name, profile=None,
             set_cookie(Cookie(
                 0, name, value, None, False,
                 domain, bool(domain), domain.startswith("."),
-                path, bool(path), secure, expires, False, None, None, {},
+                path, bool(path), secure, expires or None, False,
+                None, None, {},
             ))
 
     if failed_cookies > 0:
@@ -179,17 +179,20 @@ def _firefox_cookies_database(profile=None, container=None):
                                 "{}".format(search_root))
     _log_debug("Extracting cookies from %s", path)
 
-    if container == "none":
+    if not container or container == "none":
         container_id = False
         _log_debug("Only loading cookies not belonging to any container")
 
-    elif container:
+    elif container == "all":
+        container_id = None
+
+    else:
         containers_path = os.path.join(
             os.path.dirname(path), "containers.json")
 
         try:
-            with open(containers_path) as file:
-                identities = util.json_loads(file.read())["identities"]
+            with open(containers_path) as fp:
+                identities = util.json_loads(fp.read())["identities"]
         except OSError:
             _log_error("Unable to read Firefox container database at '%s'",
                        containers_path)
@@ -207,17 +210,17 @@ def _firefox_cookies_database(profile=None, container=None):
                 container))
         _log_debug("Only loading cookies from container '%s' (ID %s)",
                    container, container_id)
-    else:
-        container_id = None
 
     return path, container_id
 
 
 def _firefox_browser_directory():
     if sys.platform in ("win32", "cygwin"):
-        return os.path.expandvars(r"%APPDATA%\Mozilla\Firefox\Profiles")
+        return os.path.expandvars(
+            r"%APPDATA%\Mozilla\Firefox\Profiles")
     if sys.platform == "darwin":
-        return os.path.expanduser("~/Library/Application Support/Firefox")
+        return os.path.expanduser(
+            "~/Library/Application Support/Firefox/Profiles")
     return os.path.expanduser("~/.mozilla/firefox")
 
 
@@ -352,6 +355,7 @@ def _get_chromium_based_browser_settings(browser_name):
             "chromium": join(appdata_local, R"Chromium\User Data"),
             "edge"    : join(appdata_local, R"Microsoft\Edge\User Data"),
             "opera"   : join(appdata_roaming, R"Opera Software\Opera Stable"),
+            "thorium" : join(appdata_local, R"Thorium\User Data"),
             "vivaldi" : join(appdata_local, R"Vivaldi\User Data"),
         }[browser_name]
 
@@ -363,6 +367,7 @@ def _get_chromium_based_browser_settings(browser_name):
             "chromium": join(appdata, "Chromium"),
             "edge"    : join(appdata, "Microsoft Edge"),
             "opera"   : join(appdata, "com.operasoftware.Opera"),
+            "thorium" : join(appdata, "Thorium"),
             "vivaldi" : join(appdata, "Vivaldi"),
         }[browser_name]
 
@@ -375,6 +380,7 @@ def _get_chromium_based_browser_settings(browser_name):
             "chromium": join(config, "chromium"),
             "edge"    : join(config, "microsoft-edge"),
             "opera"   : join(config, "opera"),
+            "thorium" : join(config, "Thorium"),
             "vivaldi" : join(config, "vivaldi"),
         }[browser_name]
 
@@ -388,6 +394,7 @@ def _get_chromium_based_browser_settings(browser_name):
         "edge"    : "Microsoft Edge" if sys.platform == "darwin" else
                     "Chromium",
         "opera"   : "Opera" if sys.platform == "darwin" else "Chromium",
+        "thorium" : "Thorium",
         "vivaldi" : "Vivaldi" if sys.platform == "darwin" else "Chrome",
     }[browser_name]
 
@@ -680,7 +687,8 @@ def _get_gnome_keyring_password(browser_keyring_name):
     # lists all keys and presumably searches for its key in the list.
     # It appears that we must do the same.
     # https://github.com/jaraco/keyring/issues/556
-    with contextlib.closing(secretstorage.dbus_init()) as con:
+    con = secretstorage.dbus_init()
+    try:
         col = secretstorage.get_default_collection(con)
         label = browser_keyring_name + " Safe Storage"
         for item in col.get_all_items():
@@ -689,6 +697,8 @@ def _get_gnome_keyring_password(browser_keyring_name):
         else:
             _log_error("Failed to read from GNOME keyring")
             return b""
+    finally:
+        con.close()
 
 
 def _get_linux_keyring_password(browser_keyring_name, keyring):
@@ -740,8 +750,8 @@ def _get_windows_v10_key(browser_root):
         _log_error("Unable to find Local State file")
         return None
     _log_debug("Found Local State file at '%s'", path)
-    with open(path, encoding="utf-8") as file:
-        data = util.json_loads(file.read())
+    with open(path, encoding="utf-8") as fp:
+        data = util.json_loads(fp.read())
     try:
         base64_key = data["os_crypt"]["encrypted_key"]
     except KeyError:
@@ -855,7 +865,7 @@ class DatabaseConnection():
 
 
 def Popen_communicate(*args):
-    proc = subprocess.Popen(
+    proc = util.Popen(
         args, stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
     try:
         stdout, stderr = proc.communicate()
@@ -997,6 +1007,12 @@ def _decrypt_windows_dpapi(ciphertext):
 
 
 def _find_most_recently_used_file(root, filename):
+    # if the provided root points to an exact profile path
+    # check if it contains the wanted filename
+    first_choice = os.path.join(root, filename)
+    if os.path.exists(first_choice):
+        return first_choice
+
     # if there are multiple browser profiles, take the most recently used one
     paths = []
     for curr_root, dirs, files in os.walk(root):
