@@ -42,8 +42,9 @@ class YoutubeDLDownloader(DownloaderBase):
             if not ytdl_instance:
                 try:
                     module = ytdl.import_module(self.config("module"))
-                except ImportError as exc:
-                    self.log.error("Cannot import module '%s'", exc.name)
+                except (ImportError, SyntaxError) as exc:
+                    self.log.error("Cannot import module '%s'",
+                                   getattr(exc, "name", ""))
                     self.log.debug("", exc_info=True)
                     self.download = lambda u, p: False
                     return False
@@ -64,6 +65,8 @@ class YoutubeDLDownloader(DownloaderBase):
             try:
                 info_dict = ytdl_instance.extract_info(url[5:], download=False)
             except Exception:
+                pass
+            if not info_dict:
                 return False
 
         if "entries" in info_dict:
@@ -87,6 +90,11 @@ class YoutubeDLDownloader(DownloaderBase):
         formats = info_dict.get("requested_formats")
         if formats and not compatible_formats(formats):
             info_dict["ext"] = "mkv"
+        elif "ext" not in info_dict:
+            try:
+                info_dict["ext"] = info_dict["formats"][0]["ext"]
+            except LookupError:
+                info_dict["ext"] = "mp4"
 
         if self.outtmpl:
             self._set_outtmpl(ytdl_instance, self.outtmpl)
@@ -98,6 +106,7 @@ class YoutubeDLDownloader(DownloaderBase):
                 pathfmt.realdirectory + filename)
         else:
             pathfmt.set_extension(info_dict["ext"])
+            pathfmt.build_path()
 
         if pathfmt.exists():
             pathfmt.temppath = ""
@@ -118,6 +127,7 @@ class YoutubeDLDownloader(DownloaderBase):
 
     def _download_playlist(self, ytdl_instance, pathfmt, info_dict):
         pathfmt.set_extension("%(playlist_index)s.%(ext)s")
+        pathfmt.build_path()
         self._set_outtmpl(ytdl_instance, pathfmt.realpath)
 
         for entry in info_dict["entries"]:
