@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2018-2022 Mike Fährmann
+# Copyright 2018-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -32,10 +32,12 @@ class UgoiraPP(PostProcessor):
         self.extension = options.get("extension") or "webm"
         self.args = options.get("ffmpeg-args") or ()
         self.twopass = options.get("ffmpeg-twopass", False)
-        self.output = options.get("ffmpeg-output", True)
+        self.output = options.get("ffmpeg-output", "error")
         self.delete = not options.get("keep-files", False)
         self.repeat = options.get("repeat-last-frame", True)
         self.mtime = options.get("mtime", True)
+        self.skip = options.get("skip", True)
+        self.uniform = self._convert_zip = self._convert_files = False
 
         ffmpeg = options.get("ffmpeg-location")
         self.ffmpeg = util.expand_path(ffmpeg) if ffmpeg else "ffmpeg"
@@ -63,7 +65,9 @@ class UgoiraPP(PostProcessor):
         self.log.debug("using %s demuxer", demuxer)
 
         rate = options.get("framerate", "auto")
-        if rate != "auto":
+        if rate == "uniform":
+            self.uniform = True
+        elif rate != "auto":
             self.calculate_framerate = lambda _: (None, rate)
 
         if options.get("libx264-prevent-odd", True):
@@ -81,31 +85,50 @@ class UgoiraPP(PostProcessor):
         else:
             self.prevent_odd = False
 
-        job.register_hooks(
-            {"prepare": self.prepare, "file": self.convert}, options)
+        self.args_pp = args = []
+        if isinstance(self.output, str):
+            args += ("-hide_banner", "-loglevel", self.output)
+        if self.prevent_odd:
+            args += ("-vf", "crop=iw-mod(iw\\,2):ih-mod(ih\\,2)")
+
+        job.register_hooks({
+            "prepare": self.prepare,
+            "file"   : self.convert_zip,
+            "after"  : self.convert_files,
+        }, options)
 
     def prepare(self, pathfmt):
-        self._frames = None
-
-        if pathfmt.extension != "zip":
+        if "frames" not in pathfmt.kwdict:
+            self._frames = None
             return
 
-        if "frames" in pathfmt.kwdict:
-            self._frames = pathfmt.kwdict["frames"]
-        elif "pixiv_ugoira_frame_data" in pathfmt.kwdict:
-            self._frames = pathfmt.kwdict["pixiv_ugoira_frame_data"]["data"]
+        self._frames = pathfmt.kwdict["frames"]
+        if pathfmt.extension == "zip":
+            self._convert_zip = True
+            if self.delete:
+                pathfmt.set_extension(self.extension)
+                pathfmt.build_path()
         else:
-            return
+            pathfmt.build_path()
+            index = pathfmt.kwdict["_ugoira_frame_index"]
+            frame = self._frames[index].copy()
+            frame["index"] = index
+            frame["path"] = pathfmt.realpath
+            frame["ext"] = pathfmt.kwdict["extension"]
 
-        if self.delete:
-            pathfmt.set_extension(self.extension)
+            if not index:
+                self._files = [frame]
+            else:
+                self._files.append(frame)
+                if len(self._files) >= len(self._frames):
+                    self._convert_files = True
 
-    def convert(self, pathfmt):
-        if not self._frames:
+    def convert_zip(self, pathfmt):
+        if not self._convert_zip:
             return
+        self._convert_zip = False
 
         with tempfile.TemporaryDirectory() as tempdir:
-            # extract frames
             try:
                 with zipfile.ZipFile(pathfmt.temppath) as zfile:
                     zfile.extractall(tempdir)
@@ -113,49 +136,93 @@ class UgoiraPP(PostProcessor):
                 pathfmt.realpath = pathfmt.temppath
                 return
 
-            # process frames and collect command-line arguments
-            pathfmt.set_extension(self.extension)
-            args = self._process(pathfmt, tempdir)
-            if self.args:
-                args += self.args
-
-            # ensure target directory exists
-            os.makedirs(pathfmt.realdirectory, exist_ok=True)
-
-            # invoke ffmpeg
-            try:
-                if self.twopass:
-                    if "-f" not in self.args:
-                        args += ("-f", self.extension)
-                    args += ("-passlogfile", tempdir + "/ffmpeg2pass", "-pass")
-                    self._exec(args + ["1", "-y", os.devnull])
-                    self._exec(args + ["2", pathfmt.realpath])
-                else:
-                    args.append(pathfmt.realpath)
-                    self._exec(args)
-                if self._finalize:
-                    self._finalize(pathfmt, tempdir)
-            except OSError as exc:
-                print()
-                self.log.error("Unable to invoke FFmpeg (%s: %s)",
-                               exc.__class__.__name__, exc)
-                pathfmt.realpath = pathfmt.temppath
-            except Exception:
-                pathfmt.realpath = pathfmt.temppath
-            else:
-                if self.mtime:
-                    mtime = pathfmt.kwdict.get("_mtime")
-                    if mtime:
-                        util.set_mtime(pathfmt.realpath, mtime)
+            if self.convert(pathfmt, tempdir):
                 if self.delete:
                     pathfmt.delete = True
                 else:
+                    self.log.info(pathfmt.filename)
                     pathfmt.set_extension("zip")
+                    pathfmt.build_path()
+
+    def convert_files(self, pathfmt):
+        if not self._convert_files:
+            return
+        self._convert_files = False
+
+        with tempfile.TemporaryDirectory() as tempdir:
+            for frame in self._files:
+
+                # update frame filename extension
+                frame["file"] = name = "{}.{}".format(
+                    frame["file"].partition(".")[0], frame["ext"])
+
+                # move frame into tempdir
+                try:
+                    self._copy_file(frame["path"], tempdir + "/" + name)
+                except OSError as exc:
+                    self.log.debug("Unable to copy frame %s (%s: %s)",
+                                   name, exc.__class__.__name__, exc)
+                    return
+
+            pathfmt.kwdict["num"] = 0
+            self._frames = self._files
+            if self.convert(pathfmt, tempdir):
+                self.log.info(pathfmt.filename)
+                if self.delete:
+                    self.log.debug("Deleting frames")
+                    for frame in self._files:
+                        util.remove_file(frame["path"])
+
+    def convert(self, pathfmt, tempdir):
+        pathfmt.set_extension(self.extension)
+        pathfmt.build_path()
+        if self.skip and pathfmt.exists():
+            return True
+
+        # process frames and collect command-line arguments
+        args = self._process(pathfmt, tempdir)
+        if self.args_pp:
+            args += self.args_pp
+        if self.args:
+            args += self.args
+
+        # ensure target directory exists
+        os.makedirs(pathfmt.realdirectory, exist_ok=True)
+
+        # invoke ffmpeg
+        try:
+            if self.twopass:
+                if "-f" not in self.args:
+                    args += ("-f", self.extension)
+                args += ("-passlogfile", tempdir + "/ffmpeg2pass", "-pass")
+                self._exec(args + ["1", "-y", os.devnull])
+                self._exec(args + ["2", pathfmt.realpath])
+            else:
+                args.append(pathfmt.realpath)
+                self._exec(args)
+            if self._finalize:
+                self._finalize(pathfmt, tempdir)
+        except OSError as exc:
+            print()
+            self.log.error("Unable to invoke FFmpeg (%s: %s)",
+                           exc.__class__.__name__, exc)
+            pathfmt.realpath = pathfmt.temppath
+        except Exception as exc:
+            print()
+            self.log.error("%s: %s", exc.__class__.__name__, exc)
+            self.log.debug("", exc_info=True)
+            pathfmt.realpath = pathfmt.temppath
+        else:
+            if self.mtime:
+                mtime = pathfmt.kwdict.get("_mtime")
+                if mtime:
+                    util.set_mtime(pathfmt.realpath, mtime)
+            return True
 
     def _exec(self, args):
         self.log.debug(args)
         out = None if self.output else subprocess.DEVNULL
-        retcode = subprocess.Popen(args, stdout=out, stderr=out).wait()
+        retcode = util.Popen(args, stdout=out, stderr=out).wait()
         if retcode:
             print()
             self.log.error("Non-zero exit status when running %s (%s)",
@@ -163,6 +230,9 @@ class UgoiraPP(PostProcessor):
             raise ValueError()
         return retcode
 
+    def _copy_file(self, src, dst):
+        shutil.copyfile(src, dst)
+
     def _process_concat(self, pathfmt, tempdir):
         rate_in, rate_out = self.calculate_framerate(self._frames)
         args = [self.ffmpeg, "-f", "concat"]
@@ -248,8 +318,8 @@ class UgoiraPP(PostProcessor):
         append("")
 
         ffconcat = tempdir + "/ffconcat.txt"
-        with open(ffconcat, "w") as file:
-            file.write("\n".join(content))
+        with open(ffconcat, "w") as fp:
+            fp.write("\n".join(content))
         return ffconcat
 
     def _write_mkvmerge_timecodes(self, tempdir):
@@ -264,15 +334,20 @@ class UgoiraPP(PostProcessor):
         append("")
 
         timecodes = tempdir + "/timecodes.tc"
-        with open(timecodes, "w") as file:
-            file.write("\n".join(content))
+        with open(timecodes, "w") as fp:
+            fp.write("\n".join(content))
         return timecodes
 
     def calculate_framerate(self, frames):
-        uniform = self._delay_is_uniform(frames)
-        if uniform:
+        if self._delay_is_uniform(frames):
             return ("1000/{}".format(frames[0]["delay"]), None)
-        return (None, "1000/{}".format(self._delay_gcd(frames)))
+
+        if not self.uniform:
+            gcd = self._delay_gcd(frames)
+            if gcd >= 10:
+                return (None, "1000/{}".format(gcd))
+
+        return (None, None)
 
     @staticmethod
     def _delay_gcd(frames):
